@@ -3,6 +3,7 @@ local re         = require 'parser.relabel'
 local lines      = require 'parser.lines'
 local guide      = require 'parser.guide'
 local grammar    = require 'parser.grammar'
+local util       = require 'utility'
 
 local TokenTypes, TokenStarts, TokenFinishs, TokenContents
 local Ci, Offset, pushError, Ct, NextComment, Lines
@@ -1172,21 +1173,25 @@ local function buildLuaDoc(comment)
 end
 
 ---当前行在注释doc前是否有代码
-local function haveCodeBeforeDocInCurLine(lineData, docStartCol)
-    return docStartCol > lineData.sp + lineData.tab + 3
+local function haveCodeBeforeDocInCurLine(text, lineData, docStart)
+    return text:sub(lineData.start + 1, docStart - 1):find '[%w_]'
 end
 
-local function isNextLine(lns, binded, doc)
+local function isTailComment(lns, text, binded, doc)
+    local lastDoc = binded[#binded]
+    local lastDocStartRow = guide.positionOf(lns, lastDoc.originalComment.start)
+    local lastDocStartLineData = guide.lineData(lns, lastDocStartRow)
+    if haveCodeBeforeDocInCurLine(text, lastDocStartLineData, lastDoc.originalComment.start) then
+        return true
+    end
+    return false
+end
+
+local function isNextLine(lns, text, binded, doc)
     if not binded then
         return false
     end
     local lastDoc = binded[#binded]
-    local lastDocStartRow, lastDocStartCol = guide.positionOf(lns, lastDoc.originalComment.start)
-    local lastDocStartLineData = guide.lineData(lns, lastDocStartRow)
-    if haveCodeBeforeDocInCurLine(lastDocStartLineData, lastDocStartCol) then
-        return false
-    end
-
     local lastRow = guide.positionOf(lns, lastDoc.finish)
     local newRow  = guide.positionOf(lns, doc.start)
     return newRow - lastRow == 1
@@ -1339,39 +1344,47 @@ local function bindDoc(sources, lns, binded)
     bindClassAndFields(binded)
 end
 
+local bindDocAccept = {
+    'local'     , 'setlocal'  , 'setglobal',
+    'setfield'  , 'setmethod' , 'setindex' ,
+    'tablefield', 'tableindex',
+    'function'  , 'table'     , '...'      ,
+}
+
 local function bindDocs(state)
+    tracy.ZoneBeginN('bindDocs #1')
+    local text = state.lua
     local sources = {}
-    guide.eachSource(state.ast, function (src)
-        if src.type == 'local'
-        or src.type == 'setlocal'
-        or src.type == 'setglobal'
-        or src.type == 'setfield'
-        or src.type == 'setmethod'
-        or src.type == 'setindex'
-        or src.type == 'tablefield'
-        or src.type == 'tableindex'
-        or src.type == 'function'
-        or src.type == 'table'
-        or src.type == '...' then
-            sources[#sources+1] = src
-        end
+    guide.eachSourceTypes(state.ast, bindDocAccept, function (src)
+        sources[#sources+1] = src
     end)
+    tracy.ZoneEnd()
+    tracy.ZoneBeginN('bindDocs #2')
     table.sort(sources, function (a, b)
         return a.start < b.start
     end)
+    tracy.ZoneEnd()
+    tracy.ZoneBeginN('bindDocs #3')
     local binded
     for _, doc in ipairs(state.ast.docs) do
-        if not isNextLine(Lines, binded, doc) then
+        if not isNextLine(Lines, text, binded, doc) then
             bindDoc(sources, Lines, binded)
             binded = {}
             state.ast.docs.groups[#state.ast.docs.groups+1] = binded
         end
         binded[#binded+1] = doc
+        if isTailComment(Lines, text, binded, doc) then
+            bindDoc(sources, Lines, binded)
+            binded = nil
+        end
     end
     bindDoc(sources, Lines, binded)
+    tracy.ZoneEnd()
 end
 
 return function (_, state)
+    tracy.ZoneBeginN('luadoc')
+    local _ <close> = tracy.ZoneEnd
     local ast = state.ast
     local comments = state.comms
     table.sort(comments, function (a, b)
